//------------------------------------------------------------------------------
//
//	 ArchitectureCraft - Shape enum
//
//------------------------------------------------------------------------------

package gcewing.architecture;

import java.util.*;
import static java.lang.Math.*;

import net.minecraft.entity.player.*;
import net.minecraft.util.*;

import static net.minecraft.util.EnumFacing.*;

import static gcewing.architecture.ShapeKind.*;
import static gcewing.architecture.ShapeSymmetry.*;
import static gcewing.architecture.WindowShapeKinds.*;
import static gcewing.architecture.Profile.Generic.*;

public enum Shape {

	//NoShape(-1, "No Shape", Unilateral, 0, 0),

	RoofTile(0, "Roof Tile", Roof, Bilateral, 1, 2, 0xcf),
	RoofOuterCorner(1, "Roof Outer Corner", Roof, Unilateral, 1, 3, 0x4f),
	RoofInnerCorner(2, "Roof Inner Corner", Roof, Unilateral, 2, 3, 0xdf),
	RoofRidge(3, "Gabled Roof Ridge", Roof, Bilateral, 1, 4, 0x0f),
	RoofSmartRidge(4, "Hip Roof Ridge", Roof, Quadrilateral, 1, 2, 0x0f),
	RoofValley(5, "Gabled Roof Valley", Roof, Bilateral, 1, 2, 0xff),
	RoofSmartValley(6, "Hip Roof Valley", Roof, Quadrilateral, 1, 1, 0xff),
	
	RoofOverhang(7, "Roof Overhang", Model("roof_overhang"), Bilateral, 1, 2, 0xcf),
	RoofOverhangOuterCorner(8, "Roof Overhang Outer Corner", Model("roof_overhang_outer_corner"), Unilateral, 1, 3, 0x4f),
	RoofOverhangInnerCorner(9, "Roof Overhang Inner Corner", Model("roof_overhang_inner_corner"), Unilateral, 2, 3, 0xdf),
	
	RoofOverhangGableLH(40, "Gable Overhang LH", Model("roof_overhang_gable_lh"), Bilateral, 1, 4, 0x48),
	RoofOverhangGableRH(41, "Gable Overhang RH", Model("roof_overhang_gable_rh"), Bilateral, 1, 4, 0x84),
	RoofOverhangGableEndLH(42, "Gable Overhang LH End", Model("roof_overhang_gable_end_lh"), Bilateral, 1, 4, 0x48),
	RoofOverhangGableEndRH(43, "Gable Overhang RH End", Model("roof_overhang_gable_end_rh"), Bilateral, 1, 4, 0x48),
	RoofOverhangRidge(44, "Ridge Overhang", Model("roof_overhang_gable_ridge"), Bilateral, 1, 4, 0x0c),
	RoofOverhangValley(45, "Valley Overhang", Model("roof_overhang_gable_valley"), Bilateral, 1, 4, 0xcc),
	
	Cylinder(10, "Cylinder", Model("cylinder_full_r8h16"), Quadrilateral, 1, 1, 0xff),
	CylinderHalf(11, "Half Cylinder", Model("cylinder_half_r8h16"), Bilateral, 1, 1, 0xcc),
	CylinderQuarter(12, "Quarter Cylinder", Model("cylinder_quarter_r8h16"), Unilateral, 1, 1, 0x44),
	CylinderLargeQuarter(13, "Round Outer Corner", Model("cylinder_quarter_r16h16"), Unilateral, 1, 1, 0xff),
	AnticylinderLargeQuarter(14, "Round inner corner", Model("round_inner_corner"), Unilateral, 1, 2, 0xdd),
	Pillar(15, "Round Pillar", Model("cylinder_r6h16"), Quadrilateral, 1, 1, 0x106),
	Post(16, "Round Post", Model("cylinder_r4h16"), Quadrilateral, 1, 4, 0x104),
	Pole(17, "Round Pole", Model("cylinder_r2h16"), Quadrilateral, 1, 16, 0x102),
	//?
	
	PillarBase(20, "Round Pillar Base", Model("pillar_base"), Quadrilateral, 1, 1, 0xff),
	DoricCapital(21, "Doric Capital", Model("doric_capital"), Quadrilateral, 1, 1, 0xff),
	IonicCapital(22, "Ionic capital", Model("ionic_capital"), Bilateral, 1, 1, 0xff),
	CorinthianCapital(23, "Corinthian capital", Model("corinthian_capital"), Quadrilateral, 1, 1, 0xff),
	DoricTriglyph(24, "Triglyph", Model("doric_triglyph", lrStraight), Bilateral, 1, 1, 0xff),
	DoricTriglyphCorner(25, "Triglyph Corner", Model("doric_triglyph_corner", lrCorner), Bilateral, 1, 1, 0xff),
	DoricMetope(26, "Metope", Model("doric_metope", lrStraight), Bilateral, 1, 1, 0xff),
	Architrave(27, "Architrave", Model("architrave", lrStraight), Bilateral, 1, 1, 0xff),
	ArchitraveCorner(28, "Architrave Corner", Model("architrave_corner", lrCorner), Unilateral, 1, 1, 0xff),
		
	WindowFrame(30, "Window Frame", PlainWindow, Bilateral, 1, 4, 0x202),
	WindowCorner(31, "Window Corner", CornerWindow, Unilateral, 1, 2, 0x202),
	WindowMullion(32, "Window Mullion", MullionWindow, Bilateral, 1, 2, 0x202),
	
	CorniceLH(50, "Cornice LH", Model("cornice_lh"), Bilateral, 1, 4, 0x48),
	CorniceRH(51, "Cornice RH", Model("cornice_rh"), Bilateral, 1, 4, 0x84),
	CorniceEndLH(52, "Cornice LH End", Model("cornice_end_lh"), Bilateral, 1, 4, 0x48),
	CorniceEndRH(53, "Cornice RH End", Model("cornice_end_rh"), Bilateral, 1, 4, 0x48),
	CorniceRidge(54, "Cornice Ridge", Model("cornice_ridge"), Bilateral, 1, 4, 0x0c),
	CorniceValley(55, "Cornice Valley", Model("cornice_valley"), Bilateral, 1, 4, 0xcc),
	CorniceBottom(56, "Cornice Bottom", Model("cornice_bottom"), Bilateral, 1, 4, 0x0c),

	CladdingSheet(60, "Cladding", Cladding, null, 1, 16, 0),

;

	public int id;
	public String title;
	public ShapeKind kind;
	public ShapeSymmetry symmetry;
	public int materialUsed;
	public int itemsProduced;
	public int occlusionMask;
//	public String modelName;
	
	public static Shape[] values = values();
	
	protected static Map<Integer, Shape> idMap = new HashMap<Integer, Shape>();
	
	static {
		for (Shape s : values)
			idMap.put(s.id, s);
	}
	
	public static Shape forId(int id) {
		Shape shape = idMap.get(id);
		if (shape == null)
			shape = RoofTile;
		return shape;
	}
	
//	Shape(int id, String title, ShapeKind kind, ShapeSymmetry sym, int used, int made, int occ) {
//		this(id, title, kind, sym, used, made, occ, null);
//	}
//
//	Shape(int id, String title, ShapeKind kind, ShapeSymmetry sym, int used, int made, int occ, String model)

	Shape(int id, String title, ShapeKind kind, ShapeSymmetry sym, int used, int made, int occ) {
		this.id = id;
		this.title = title;
		this.kind = kind;
		this.symmetry = sym;
		this.materialUsed = used;
		this.itemsProduced = made;
		this.occlusionMask = occ;
//		this.modelName = "shape/" + model;
	}
	
	protected void orientOnPlacement(EntityPlayer player, ShapeTE te, ShapeTE nte, EnumFacing face, Vector3 hit) {
//		if (nte != null && inheritsPlacementOrientationFrom(nte.shape)) {
//			te.setSide(nte.side);
//			te.setTurn(nte.turn);
//		}
		if (te.shape.kind.orientOnPlacement(player, te, nte, face, hit))
			return;
		else
			orientFromHitPosition(player, te, face, hit);
	}
	
	protected void orientFromHitPosition(EntityPlayer player, ShapeTE te, EnumFacing face, Vector3 hit) {
		int side, turn;
		switch (face) {
			case UP:
				side = 0;
				break;
			case DOWN:
				if (te.shape.kind.canPlaceUpsideDown())
					side = 1;
				else
					side = 0;
				break;
			default:
				if (player.isSneaking())
					side = face.getOpposite().ordinal();
				else if (hit.y > 0.0 && te.shape.kind.canPlaceUpsideDown())
					side = 1;
				else
					side = 0;
		}
		Trans3 tSide = new Trans3(0, 0, 0).side(side);
		Vector3 h = tSide.ip(hit);
		turn = te.shape.turnForPlacementHit(h.x, h.z);
//		if (!te.getWorld().isRemote) {
//			System.out.printf("Shape.orientFromHitPosition: face %s global hit %s local hit (%.3f, %.3f)\n", face, hit, h.x, h.z);
//			System.out.printf("Shape.orientFromHitPosition: side %s turn %s symmetry %s\n", side, turn, te.shape.symmetry);
//		}
		te.setSide(side);
		te.setTurn(turn);
	}
	
	protected int turnForPlacementHit(double x, double z) {
		switch (symmetry) {
			case Quadrilateral: // All rotations are equivalent
				return 0;
			case Bilateral: // Rotate according to nearest side
				if (abs(z) > abs(x))
					return z < 0 ? 2 : 0;
				else
					return x > 0 ? 1 : 3;
			case Unilateral: // Rotate according to nearest corner
				if (z > 0)
					return x < 0 ? 0 : 1;
				else
					return x > 0 ? 2 : 3;
			default:
				return 0;
		}
	}
	
//	protected boolean inheritsPlacementOrientationFrom(Shape other) {
//		return this.kind.inheritsPlacementOrientation(this, other);
//	}
	
}
